/*
** dirfind.d
** Copyright (C) 2003 Helmut Leitner
** Donated to Digital Mars.
** Released under the GNU GPL, see LICENSE.TXT
** and http://www.gnu.org/copyleft/gpl.html
*/

module venus.dirfind;
import venus.all;

const int DFF_FILES = 1;
const int DFF_DIRS = 2;
const int DFF_RECURSE = 4;

alias pavel.win32.WIN32_FIND_DATA FILEINFO;
alias pavel.win32.FILE_ATTRIBUTE_SYSTEM FILE_ATTRIBUTE_SYSTEM;
alias pavel.win32.FILE_ATTRIBUTE_HIDDEN FILE_ATTRIBUTE_HIDDEN;
alias pavel.win32.FILE_ATTRIBUTE_DIRECTORY FILE_ATTRIBUTE_DIRECTORY;
alias pavel.win32.FindFirstFileA FindFirstFileA;
alias pavel.win32.FindNextFileA FindNextFileA;

int FileInfoRetDirFlag(FILEINFO *fb) {
    int flag = 0;

    if(fb.cFileName[0] != '.' && (fb.dwFileAttributes & (FILE_ATTRIBUTE_SYSTEM | FILE_ATTRIBUTE_HIDDEN)) == 0) {
        if(fb.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
            flag = 1;
        }
    }
    return flag;
}

int FileInfoRetFileFlag(FILEINFO *fb) {
    int flag = 0;

    if(fb.cFileName[0] != '.' && (fb.dwFileAttributes & (FILE_ATTRIBUTE_SYSTEM | FILE_ATTRIBUTE_HIDDEN)) == 0) {
        if((fb.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY) == 0) {
            flag = 1;
        }
    }
    return flag;
}

int FileExtCmp(char *s1,char *s2) {
    while(*s1 && *s2) {
        if(*s1 == '*' || *s2 == '*') {
            return 0;
        }
        if(*s1 != *s2 && *s1 != '?' && *s2 != '?') {
            return -1;
        }
        s1++; s2++;
    }
    if(*s1 == '*' || *s2 == '*') {
        return 0;
    }
    if(*s1 == *s2) {
        return 0;
    }
    return -1;
}

int FileNameCmp(char [] fnam1,char [] fnam2) {
    char [][] a1 = split(string.toupper(fnam1),".");
    char [][] a2 = split(string.toupper(fnam2),".");

    if(a1.length < 2) {
        a1 ~= "";
    }
    if(a2.length < 2) {
        a2 ~= "";
    }

    if(FileExtCmp(a1[0],a2[0])) {
        return -1;
    }
    if(FileExtCmp(a1[1],a2[1])) {
        return -1;
    }

    return 0;
}

int DirFindFileCall(char [] dir, char [] fspec,int delegate (char [] d,char [] fn,FILEINFO *b) f, int flags){
    FILEINFO b;
    int done = false;
    int err = 0;
    char [] fnam2;
    char [] dir2;

    pavel.win32.HANDLE hd=FindFirstFileA(dir ~ "*.*", &b);
    while((hd != pavel.win32.INVALID_HANDLE_VALUE) && (done == false))
    {
        fnam2 = string.toString(b.cFileName); // !!! has me bitten
        //printf("%.*s\n",fnam2);
        if(FileInfoRetFileFlag(&b)) {
            if(flags & DFF_FILES) {
                if(FileNameCmp(fnam2,fspec) == 0) {
                    if(f(dir,fnam2,&b)) {
                        goto do_exit;
                    }
                }
            }
        }
        if(FileInfoRetDirFlag(&b)) {
            if(flags & DFF_DIRS) {
                if(f(dir,fnam2,&b)) {
                    err = -1;
                    goto do_exit;
                }
            }
            if(flags & DFF_RECURSE) {
                dir2 = dir ~ fnam2 ~ "\\";
                if(DirFindFileCall(dir2,fspec,f,flags)) {
                    goto do_exit;
                }
            }
        }
        done = (FindNextFileA(hd,&b) == false);
    }
do_exit:
    if(hd) {
        pavel.win32.FindClose(hd);
    }
    return err;
}

char [][] DirFindFile(char [] dir,char [] fspec, int flags)
{
    char [][] files;

    DirFindFileCall(dir,fspec,
        delegate int (char [] dir, char [] fnam, FILEINFO *b)
        {
            files ~= (dir ~ fnam);
            return 0;
        }
        ,flags
    );
    return files;
}
